@@ -3,18 +3,25 @@
 # Electrum - lightweight Bitcoin client
 # Copyright (C) 2012 thomasv@gitorious
 #
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# Permission is hereby granted, free of charge, to any person
+# obtaining a copy of this software and associated documentation files
+# (the "Software"), to deal in the Software without restriction,
+# including without limitation the rights to use, copy, modify, merge,
+# publish, distribute, sublicense, and/or sell copies of the Software,
+# and to permit persons to whom the Software is furnished to do so,
+# subject to the following conditions:
 #
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
+# The above copyright notice and this permission notice shall be
+# included in all copies or substantial portions of the Software.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program. If not, see <http://www.gnu.org/licenses/>.
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
+# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
+# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
+# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+# SOFTWARE.
 
 import datetime
 import json
@@ -38,23 +45,22 @@ def show_transaction(tx, parent, desc=None, prompt_if_unsaved=False):
     dialogs.append(d)
     d.show()
 
-class TxDialog(QDialog):
+class TxDialog(QDialog, MessageBoxMixin):
 
     def __init__(self, tx, parent, desc, prompt_if_unsaved):
         '''Transactions in the wallet will show their description.
         Pass desc to give a description for txs not yet in the wallet.
         '''
+        QDialog.__init__(self, parent=None)   # Top-level window
         self.tx = tx
         self.tx.deserialize()
-        self.parent = parent
+        self.main_window = parent
         self.wallet = parent.wallet
         self.prompt_if_unsaved = prompt_if_unsaved
         self.saved = False
-        self.broadcast = False
         self.desc = desc
 
-        QDialog.__init__(self)
-        self.setMinimumWidth(600)
+        self.setMinimumWidth(660)
         self.setWindowTitle(_("Transaction"))
 
         vbox = QVBoxLayout()
@@ -62,7 +68,7 @@ class TxDialog(QDialog):
 
         vbox.addWidget(QLabel(_("Transaction ID:")))
         self.tx_hash_e  = ButtonsLineEdit()
-        qr_show = lambda: self.parent.show_qrcode(str(self.tx_hash_e.text()), 'Transaction ID')
+        qr_show = lambda: parent.show_qrcode(str(self.tx_hash_e.text()), 'Transaction ID', parent=self)
         self.tx_hash_e.addButton(":icons/qrcode.png", qr_show, _("Show as QR code"))
         self.tx_hash_e.setReadOnly(True)
         vbox.addWidget(self.tx_hash_e)
@@ -99,7 +105,7 @@ class TxDialog(QDialog):
         b.setIcon(QIcon(":icons/qrcode.png"))
         b.clicked.connect(self.show_qr)
 
-        self.copy_button = CopyButton(lambda: str(self.tx), self.parent.app)
+        self.copy_button = CopyButton(lambda: str(self.tx), parent.app)
 
         # Action buttons
         self.buttons = [self.sign_button, self.broadcast_button, self.cancel_button]
@@ -116,26 +122,27 @@ class TxDialog(QDialog):
         self.update()
 
     def do_broadcast(self):
-        self.parent.broadcast_transaction(self.tx, self.desc, parent=self)
-        self.broadcast = True
+        self.main_window.push_top_level_window(self)
+        try:
+            self.main_window.broadcast_transaction(self.tx, self.desc)
+        finally:
+            self.main_window.pop_top_level_window(self)
+        self.saved = True
         self.update()
 
     def closeEvent(self, event):
-        if (self.prompt_if_unsaved and not self.saved and not self.broadcast
-            and QMessageBox.question(
-                self, _('Warning'),
-                _('This transaction is not saved. Close anyway?'),
-                QMessageBox.Yes | QMessageBox.No) == QMessageBox.No):
+        if (self.prompt_if_unsaved and not self.saved
+            and not self.question(_('This transaction is not saved. Close anyway?'), title=_("Warning"))):
             event.ignore()
         else:
             event.accept()
             dialogs.remove(self)
 
     def show_qr(self):
-        text = self.tx.raw.decode('hex')
+        text = str(self.tx).decode('hex')
         text = base_encode(text, base=43)
         try:
-            self.parent.show_qrcode(text, 'Transaction')
+            self.main_window.show_qrcode(text, 'Transaction', parent=self)
         except Exception as e:
             self.show_message(str(e))
 
@@ -143,18 +150,19 @@ class TxDialog(QDialog):
     def sign(self):
         def sign_done(success):
             self.sign_button.setDisabled(False)
-            self.prompt_if_unsaved = False
-            self.saved = False
-            self.update()
-        self.sign_button.setDisabled(True)
-        cancelled, ret = self.parent.sign_tx(self.tx, sign_done, parent=self)
-        if cancelled:
-            self.sign_button.setDisabled(False)
+            self.main_window.pop_top_level_window(self)
+            if success:
+                self.prompt_if_unsaved = True
+                self.saved = False
+                self.update()
 
+        self.sign_button.setDisabled(True)
+        self.main_window.push_top_level_window(self)
+        self.main_window.sign_tx(self.tx, sign_done)
 
     def save(self):
         name = 'signed_%s.txn' % (self.tx.hash()[0:8]) if self.tx.is_complete() else 'unsigned.txn'
-        fileName = self.parent.getSaveFileName(_("Select where to save your signed transaction"), name, "*.txn")
+        fileName = self.main_window.getSaveFileName(_("Select where to save your signed transaction"), name, "*.txn")
         if fileName:
             with open(fileName, "w+") as f:
                 f.write(json.dumps(self.tx.as_dict(), indent=4) + '\n')
@@ -173,7 +181,7 @@ class TxDialog(QDialog):
             status = _("Signed")
 
             if tx_hash in self.wallet.transactions.keys():
-                desc, is_default = self.wallet.get_label(tx_hash)
+                desc = self.wallet.get_label(tx_hash)
                 conf, timestamp = self.wallet.get_confirmations(tx_hash)
                 if timestamp:
                     time_str = datetime.datetime.fromtimestamp(timestamp).isoformat(' ')[:-3]
@@ -183,7 +191,7 @@ class TxDialog(QDialog):
             else:
                 self.broadcast_button.show()
                 # cannot broadcast when offline
-                if self.parent.network is None:
+                if self.main_window.network is None:
                     self.broadcast_button.setEnabled(False)
         else:
             s, r = self.tx.signature_count()
@@ -213,16 +221,19 @@ class TxDialog(QDialog):
         if not self.wallet.up_to_date:
             return
 
+        base_unit = self.main_window.base_unit()
+        format_amount = self.main_window.format_amount
+
         if is_relevant:
             if is_mine:
                 if fee is not None:
-                    self.amount_label.setText(_("Amount sent:")+' %s'% self.parent.format_amount(-v+fee) + ' ' + self.parent.base_unit())
-                    self.fee_label.setText(_("Transaction fee")+': %s'% self.parent.format_amount(-fee) + ' ' + self.parent.base_unit())
+                    self.amount_label.setText(_("Amount sent:")+' %s'% format_amount(-v+fee) + ' ' + base_unit)
+                    self.fee_label.setText(_("Transaction fee")+': %s'% format_amount(-fee) + ' ' + base_unit)
                 else:
-                    self.amount_label.setText(_("Amount sent:")+' %s'% self.parent.format_amount(-v) + ' ' + self.parent.base_unit())
+                    self.amount_label.setText(_("Amount sent:")+' %s'% format_amount(-v) + ' ' + base_unit)
                     self.fee_label.setText(_("Transaction fee")+': '+ _("unknown"))
             else:
-                self.amount_label.setText(_("Amount received:")+' %s'% self.parent.format_amount(v) + ' ' + self.parent.base_unit())
+                self.amount_label.setText(_("Amount received:")+' %s'% format_amount(v) + ' ' + base_unit)
         else:
             self.amount_label.setText(_("Transaction unrelated to your wallet"))
 
@@ -234,7 +245,7 @@ class TxDialog(QDialog):
         if self.tx.locktime > 0:
             vbox.addWidget(QLabel("LockTime: %d\n" % self.tx.locktime))
 
-        vbox.addWidget(QLabel(_("Inputs") + ' (%d)'%len(self.tx.inputs)))
+        vbox.addWidget(QLabel(_("Inputs") + ' (%d)'%len(self.tx.inputs())))
 
         ext = QTextCharFormat()
         rec = QTextCharFormat()
@@ -249,12 +260,15 @@ class TxDialog(QDialog):
                 return chg if self.wallet.is_change(addr) else rec
             return ext
 
+        def format_amount(amt):
+            return self.main_window.format_amount(amt, whitespaces = True)
+
         i_text = QTextEdit()
         i_text.setFont(QFont(MONOSPACE_FONT))
         i_text.setReadOnly(True)
         i_text.setMaximumHeight(100)
         cursor = i_text.textCursor()
-        for x in self.tx.inputs:
+        for x in self.tx.inputs():
             if x.get('is_coinbase'):
                 cursor.insertText('coinbase')
             else:
@@ -270,10 +284,12 @@ class TxDialog(QDialog):
                 if addr is None:
                     addr = _('unknown')
                 cursor.insertText(addr, text_format(addr))
+                if x.get('value'):
+                    cursor.insertText(format_amount(x['value']), ext)
             cursor.insertBlock()
 
         vbox.addWidget(i_text)
-        vbox.addWidget(QLabel(_("Outputs") + ' (%d)'%len(self.tx.outputs)))
+        vbox.addWidget(QLabel(_("Outputs") + ' (%d)'%len(self.tx.outputs())))
         o_text = QTextEdit()
         o_text.setFont(QFont(MONOSPACE_FONT))
         o_text.setReadOnly(True)
@@ -283,11 +299,6 @@ class TxDialog(QDialog):
             cursor.insertText(addr, text_format(addr))
             if v is not None:
                 cursor.insertText('\t', ext)
-                cursor.insertText(self.parent.format_amount(v, whitespaces = True), ext)
+                cursor.insertText(format_amount(v), ext)
             cursor.insertBlock()
         vbox.addWidget(o_text)
-
-
-
-    def show_message(self, msg):
-        QMessageBox.information(self, _('Message'), msg, _('OK'))
